function gammas = grid_opt(tt_data, opt_options)
%UNTITLED3 Summary of this function goes here
%   Detailed explanation goes here

k  = opt_options.kernel;
kp = opt_options.kernelparam;
nn = opt_options.nearestneighbors;
cl = opt_options.classifier;

options = ml_options('Kernel', k, 'KernelParam', kp, 'NN', nn);

%{
  Basic idea: we have two parameters, gamma_A (labeled samples)
  and gamma_I (unlabeled ones). We have to optimize on this two parameters.
  The first step is to optimize on a grid ranging from [gA_begin, gI_begin]
  to [gA_end, gI_end], with two granularities (gA_interval and gI_interval)
  resulting from the specified number of iterations (gA_max_steps, gI_max_steps).
  For each point on the grid we train with the selected gammas, and
  subsequently test. The outcome is a scalar error, that we try to minimize.
  Once we find the "raw global minimum", we try using something smarter,
  such a gradient descent starting from the selected pair (gA, gI).
%}

gA_begin = 0.0001;
gI_begin = 0.0001;

if strcmp(opt_options.mode, 'unsupervised')
    gA_end   = 5;
    gI_end   = 100;
else
    gA_end   = 100;
    gI_end   = 100;
end

%gA_end   = 0.01;
%gI_end   = 0.0006;
%Min Err =  1.450 gA =  0.095000 gI =  0.000160

gA_max_steps = 10;
gI_max_steps = 10;

gA_interval = (gA_end - gA_begin) / gA_max_steps;
gI_interval = (gI_end - gI_begin) / gI_max_steps;

options.gamma_A = gA_begin;
options.gamma_I = gI_begin;

%{
options.NN          = 6;
options.Kernel      = 'rbf';
options.KernelParam = 0.35;

handles.data = load('2moons.mat');
handles.mode=2; % fully supervised
handles.kernel='rbf';
handles.kernelparam=0.35;
handles.algo='rlsc';
handles.lab=2;
handles.generate=1;
handles.Y=handles.data.y;
%}

%options             = ml_options('Kernel','rbf','KernelParam',0.35,'NN',6);
%[X,Y]               = generate_toydata(1000,'moons');
%[XT,YT]             = generate_toydata(100 ,'moons');
%classifier          = ml_train(XT,YT,options,'lapsvm');

min_err = 1000;
%found = 0;
%max_iter = 100;

%%%%%%%%%%%%%%%
% Grid search %
%%%%%%%%%%%%%%%

gA = 0.0001;
gI = 0.0001;

for i = 0:gA_max_steps
    for j = 1:gI_max_steps
    
        classifier            = ml_train (tt_data.XT, tt_data.YT, options, cl);
        
        [f,labels,test_error] = ml_test  (classifier, tt_data.X, tt_data.Y);
        
        %opt_options.mode
        fprintf('%s %5d\t%s %.3f\n', 'Step=', i*gA_max_steps+j, 'Error=', test_error);
        
        %{
        disp('Iter = ');
        disp(j);
        disp('Err = ');
        disp(test_error);
        %}
        
        if (min_err > test_error)
            min_err = test_error;
            gA = options.gamma_A;
            gI = options.gamma_I;
            
            fprintf('%s %.3f %s %.6f %s %.6f\n', 'MinErr=', min_err, 'gA=', gA, 'gI=', gI);
            
            %{
            disp('Min_err = ');
            disp(min_err);
            disp('gA = ');
            disp(gA);
            disp('gI = ');
            disp(gI);
            %}
        end
        
        %break;
        
        %disp(options.gamma_A);
        %disp(options.gamma_I);
        options.gamma_A = gA_begin + i * gA_interval;
        options.gamma_I = gI_begin + j * gI_interval;
        %disp(options.gamma_A);
        %disp(options.gamma_I);
        %{
        if (j == 1)
            options.gamma_A = 0.014;
            options.gamma_I = 0.785;
        end
        if (j == 2)
            options.gamma_A = 0.281;
            options.gamma_I = 6.644;
        end
        %}
            
        %if (min_err <= 1) found = 1; end
        %break;
    end
    %break;
end

gammas.gA = gA;
gammas.gI = gI;

nerr = find(tt_data.Y - labels);
%disp(nerr');
%disp(size(nerr));

%fprintf('%s %d\n', 'Size nErr = ' , size(nerr));
%fprintf('%s %d\n', 'Size nErr = ', size(nerr));

fprintf('\n');

fprintf('%s', 'Size nErr = ');
fprintf('%d', round(size(nerr)));

%fprintf('%s %d', 'Size nErr = ', round(size(nerr)));

fprintf('\n\n');

fprintf('%s %.3f %s %.6f %s %.6f\n', 'MinErr=', min_err, 'gA=', gA, 'gI=', gI);

end

